
var g_plines = [];
var g_factories = [];

function IsPolyline( crv )
{
	var segs = crv.getSubObjects();

	if ( segs.length == 1 )
		return false;

	for ( var i = 0; i < segs.length; ++i )
	{
		var seg = segs.item(i);
		if ( !seg.isLine )
			return false;
	}

	return true;
}

function CancelFactories()
{
	for ( var i = 0; i < g_factories.length; ++i )
		g_factories[i].cancel();
}

function CommitFactories()
{
	for ( var i = 0; i < g_factories.length; ++i )
		g_factories[i].commit();
}

function HidePolylines()
{
	for ( var i = 0; i < g_plines.length; ++i )
		g_plines[i].hidden = true;
}

function ShowPolylines()
{
	for ( var i = 0; i < g_plines.length; ++i )
		g_plines[i].hidden = false;
}

function RemovePolylines()
{
	for ( var i = 0; i < g_plines.length; ++i )
		moi.geometryDatabase.removeObject( g_plines[i] );
}

function GetPoints( pline, segs, seg_index, radius )
{
	if ( seg_index == 0 && !pline.isClosed )
		return [];

	var this_seg = segs.item( seg_index );

	if ( seg_index == 0 )
		prev_seg = segs.item( segs.length-1 );
	else
		prev_seg = segs.item( seg_index-1 );

	var points = [];


	var prev_tan = prev_seg.evaluateTangent( prev_seg.domainMax );
	prev_tan.scale( -radius );

	var prev_pt = moi.vectorMath.add( this_seg.getStartPt(), prev_tan );
	points.push( prev_pt );


	var this_tan = this_seg.evaluateTangent( this_seg.domainMin );
	this_tan.scale( radius );

	var this_pt = moi.vectorMath.add( this_seg.getStartPt(), this_tan );
	points.push( this_pt );

	return points;
}

function Update()
{
	CancelFactories();
	g_factories = [];


	if ( moi.ui.commandUI.mode.value == 'points' )
		ShowPolylines();
	else
		HidePolylines();

	var radius = 0.0;
	try {
		radius = moi.ui.commandUI.radius.value;
	}
	catch(e) {}

	if ( radius == 0.0 )
		return;


	for ( var i = 0; i < g_plines.length; ++i )
	{
		var pline = g_plines[i];
		var has_selected_edit_pts = pline.hasSelectedEditPoints;

		var cvs = [], corners = [];

		var segs = pline.getSubObjects();
		for ( var j = 0; j < segs.length; ++j )
		{
			if ( has_selected_edit_pts && !pline.getEditPointSelected(j) )
			{
				if ( moi.ui.commandUI.mode.value == 'curve' || moi.ui.commandUI.mode.value == 'polyline' )
				{
					cvs.push( segs.item(j).getStartPt() );
					corners.push( true );
				}
				continue;
			}

			var pts = GetPoints( pline, segs, j, radius );

			if ( moi.ui.commandUI.mode.value == 'points' )
			{
				for ( var k = 0; k < pts.length; ++k )
				{
					var factory = moi.command.createFactory( 'point' );
					factory.setInput( 0, pts[k] );
					factory.update();
					g_factories.push( factory );
				}
			}
			else
			{
				if ( pts.length == 2 )
				{
					cvs.push( pts[0] );
					corners.push( false );
				}

				cvs.push( segs.item(j).getStartPt() );
				corners.push( false );

				if ( pts.length == 2 )
				{
					cvs.push( pts[1] );
					corners.push( false );
				}
			}
		}

		if ( moi.ui.commandUI.mode.value == 'curve' || moi.ui.commandUI.mode.value == 'polyline' )
		{
			if ( pline.isClosed && cvs.length > 0 )
			{
				cvs.push( cvs[0] );
				corners.push( corners[0] );
			}

			if ( !pline.isClosed )
			{
				cvs.push( pline.getEndPt() );
				corners.push( true );
			}

			var factory = moi.command.createFactory( moi.ui.commandUI.mode.value );
			for ( var j = 0; j < cvs.length; ++j )
			{
				// Point
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs-1, cvs[j] );

				if ( moi.ui.commandUI.mode.value == 'curve' )
				{
					// Bool flag for if point is a corner point.
					factory.createInput( 'bool' );
					factory.setInput( factory.numInputs - 1, corners[j] );
				}
			}

			factory.update();
			g_factories.push( factory );

			if ( pline.showPoints )
			{
				if ( moi.ui.commandUI.mode.value == 'polyline' )
				{
					// Polyline factory has some custom handling and doesn't work with .getCreatedObjects(), so use alternate method.
					var curves = moi.geometryDatabase.getObjects().getCurves();
					for ( var j = 0; j < curves.length; ++j )
					{
						var crv = curves.item(j);
						if ( crv.databaseRevision > moi.command.lastCommandRevisionStart )
							crv.showPoints = true;
					}
				}
				else
				{
					factory.getCreatedObjects().setProperty( 'showPoints', true );
				}
			}
		}
	}
}

function OnPointUpdate( pointpicker )
{
	if ( !pointpicker.hasBasePt )
		return;

	var dist = moi.vectorMath.distance( pointpicker.basept, pointpicker.pt );
	if ( dist < 0.00001 )
		return;

	moi.ui.commandUI.radius.value = dist;

	Update();
}

function AddCornerPoints()
{
	var curves = moi.geometryDatabase.getObjects().getCurves();
	for ( var i = 0; i < curves.length; ++i )
	{
		var crv = curves.item(i);
		if ( !IsPolyline( crv ) )
			continue;

		if ( crv.selected || crv.hasSelectedEditPoints )
			g_plines.push( crv );
	}

	if ( g_plines.length == 0 )
	{
		moi.ui.alert( 'Pas de Points de Polylignes ou de Polylignes slectionnes' );
		return;
	}

	Update();

	var pointpicker = moi.ui.createPointPicker();
	pointpicker.bindFunc( OnPointUpdate );
	var pickcount = 0;

	while ( 1 )
	{
		if ( !pointpicker.waitForEvent() )
		{
			ShowPolylines();
			CancelFactories();
			return false;
		}

		if ( pointpicker.event == 'done' )
			break;

		if ( pointpicker.event == 'radius' || pointpicker.event == 'mode' )
		{
			Update();
			continue;
		}

		if ( pointpicker.event == 'finished' )
		{
			++pickcount;

			if ( pickcount & 1 )
			{
				pointpicker.setBasePt( pointpicker.pt );
			}
			else
			{
				pointpicker.clearBasePt();
				moi.ui.clearPickedPoints();
			}
		}
	}

	ShowPolylines();

	if ( moi.ui.commandUI.mode.value == 'curve' || moi.ui.commandUI.mode.value == 'polyline' )
		RemovePolylines();

	CommitFactories();
}

AddCornerPoints();
